(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.b = factory());
}(this, (function () { 'use strict';

  var el = typeof document !== 'undefined' && document.createElement('div'); // eslint-disable-line

  var cache = {
    flex: 0,
    border: 1,
    transform: 1,
    'line-height': 0,
    'box-shadow': 1,
    'border-top': 1,
    'border-left': 1,
    'border-right': 1,
    'border-bottom': 1
  };

  function px(x) {
    if ((x[0] === '-' && x[1] === '-') || x in cache)
      { return cache[x] }

    try {
      el.style[x] = '1px';
      el.style.setProperty(x, '1px');
      return cache[x] = el.style[x].slice(-3) === '1px'
    } catch (err) {
      return cache[x] = false
    }
  }

  var vendorRegex = /^(o|O|ms|MS|Ms|moz|Moz|webkit|Webkit|WebKit)([A-Z])/;

  var snake       = function (x) { return x.replace(/(\B[A-Z])/g, '-$1').toLowerCase(); };
  var findWidth   = function (x) { return x ? x.hasOwnProperty('width') ? x : findWidth(Object.getPrototypeOf(x)) : {}; };

  var properties = ['float']
    .concat(Object.keys(
      typeof document === 'undefined' ? {} : findWidth(document.documentElement.style) // eslint-disable-line
    ))
    .filter(function (x, i, xs) { return x.indexOf('-') === -1 && x !== 'length' && xs.indexOf(x) === i; })
    .map(function (x) { return x.match(vendorRegex) ? '-' + snake(x) : snake(x); })
    .sort();

  var initials = function (acc, x) {
    var obj;

    return Object.assign(acc, ( obj = {}, obj[x.split('-').map(function (x) { return x[0]; }).join('')] = x, obj ));
  };

  var popular = [
    'align-items',
    'bottom',
    'background-color',
    'border-radius',
    'box-shadow',
    'background-image',
    'color',
    'display',
    'float',
    'flex-direction',
    'font-family',
    'font-size',
    'height',
    'justify-content',
    'left',
    'line-height',
    'letter-spacing',
    'margin',
    'margin-bottom',
    'margin-left',
    'margin-right',
    'margin-top',
    'opacity',
    'padding',
    'padding-bottom',
    'padding-left',
    'padding-right',
    'padding-top',
    'right',
    'top',
    'text-align',
    'text-decoration',
    'text-transform',
    'width'
  ];

  var shorts = Object.assign(
    properties.reduce(initials, {}),
    popular.reduce(initials, {})
  );

  function shorts$1 (x) { return shorts[x] || x; }

  var doc = typeof document !== 'undefined' && window.document // eslint-disable-line
      , validChar = function (c) { return function (x) { return c.indexOf(x) > -1; }; }
      , isIdentifier = validChar('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_')
      , isQuote = validChar('\'"')
      , isValueSep = validChar(' ,)')
      , isNumber = function (x) { return typeof x === 'number'; }
      , vendorMap = Object.create(null, {})
      , vendorValuePrefix = Object.create(null, {})
      , randomId = function () { return 'bss' + ('000000' + (Math.random() * Math.pow(36, 6) | 0).toString(36)).slice(-6); };

  properties.forEach(function (x) {
    var vendor = x.match(/-(ms|o|webkit|moz)-/g);
    if (vendor) {
      var unprefixed = x.replace(/-(ms|o|webkit|moz)-/, '');
      if (properties.indexOf(unprefixed) === -1) {
        if (unprefixed === 'flexDirection')
          { vendorValuePrefix.flex = '-' + vendor[1].toLowerCase() + '-flex'; }
        vendorMap[unprefixed] = x;
        return
      }
    }
  });

  function bss(ref) {
    if ( ref === void 0 ) ref = {};
    var style = ref.style; if ( style === void 0 ) style = doc && doc.querySelector && (doc.querySelector('.bss_style') || doc.createElement('style'));
    var prefix = ref.prefix; if ( prefix === void 0 ) prefix = style && style.getAttribute('id') || randomId();
    var append = ref.append; if ( append === void 0 ) append = true;

    var cache = {}
        , anims = {}
        , props = []
        , ats = [];

    var b = bss();

    b.count = 0;
    b.rules = style && style.textContent && style.textContent.split(b.prefix + '{}') || [];
    b.debug = false;
    b.prefix = prefix;
    b.properties = properties;
    b.at = function (x) { return ats.unshift(typeof x === 'function' ? x : (function (v) { return x[v.slice(1)] || v; })); };
    b.prop = function (fn) { return props.unshift(fn); };
    b.create = bss;
    b.ssr = ssr;
    b.global = global;

    b.prop(vendor);
    b.prop(shorts$1);

    style && (style.id = b.prefix);

    return b

    function global() {
      var style = b.apply(b, arguments);
      parse(style.content.replace(/@import[^;]*;/g, insert).trim(), '__global').forEach(function (x) { return insert(x.replace(/\.__global/g, '').trim()); }
      );
    }

    function ValueOf() {
      if (this.className)
        { return this.className }

      this.className = b.prefix + ++b.count;
      var x = parse(this.content.trim(), this.className);
      x.forEach(insert);
      return x.name
    }

    function bss(pre) {
      return function(xs) {
        var args = [], len = arguments.length - 1;
        while ( len-- > 0 ) args[ len ] = arguments[ len + 1 ];

        var s = (pre || '') + xs[0];
        for (var i = 1; i < xs.length; i++) {
          var value = args[i - 1];
          s += (value && value.toString === ValueOf
            ? value.content
            : Array.isArray(value)
              ? value.map(function (x) { return x.content || x; }).join('\n')
              : value)
            + xs[i];
        }

        if (s in cache)
          { return cache[s] }

        var x = bss('\n' + s + '\n');
        x.toString = x.valueOf = ValueOf;
        x.content = s;

        cache[s] = x;

        return x
      }
    }

    function ssr() {
      return '<style class="bss_style" id="' + b.prefix + '">' + b.rules.join(b.prefix + '{}') + '</style>'
    }

    function vendor(x) {
      if (properties.indexOf(x) === -1) {
        if (vendorMap[x]) {
          b.debug && console.log(x, 'prefixed to', vendorMap[x]); // eslint-disable-line
          return vendorMap[x]
        }
        b.debug && x.indexOf('--') !== 0 && console.log(x, 'not found'); // eslint-disable-line
      }
      return x
    }

    function insert(rule) {
      if (append) {
        style && doc.head && doc.head.appendChild(style);
        append = false;
      }

      var isImport = rule.indexOf('@import') === 0
          , index = isImport ? 0 : b.rules.length;

      if (b.debug && style) {
        index === 0 ? b.rules.unshift(rule) : b.rules.push(rule);
        style.textContent = b.rules.map(pretty).join('\n');
      } else if (style && style.sheet) {
        try {
          style.sheet.insertRule(rule, index);
          index === 0 ? b.rules.unshift(rule) : b.rules.push(rule);
        } catch (e) {
          b.debug && console.log('Ignored error:', e); // eslint-disable-line
        }
      } else {
        index === 0 ? b.rules.unshift(rule) : b.rules.push(rule);
      }
    }

    function pretty(str) {
      return str
        .replace(/{/g, ' {\n')
        .replace(/([;{])\n*/g, '$1\n  ')
        .replace(/(.*):(.*);/g, '$1: $2;')
        .replace(/[\s]*}/g, '\n}\n')
        .replace(/,\s*/g, ', ')
        .replace(/@.*{[\s\S]*?}[\s]*}/g, function (m) { return m.split('\n').reduce(function (acc, x, i, xs) { return acc + (i < xs.length - 1 ? '\n' : '') + (i > 1 && i < xs.length - 2 ? '  ' : '') + x; },
            ''
          ); }
        ).trim().replace('}\n\n}', '}\n}')
    }

    function parse(s, x) {
      var rules = []
          , props = propsState()
          , level = [];

      var ani = false
        , at = false
        , supports = false
        , name = false;

      for (var i = 0; i < s.length; i++) {
        if (i > 0 && name && !isIdentifier(s[i])) {
          x = s.slice(1, i) + '-' + x;
          name = false;
        }
        if (name || (i === 0 && s[i] === '.')) {
          name = true;
          continue // eslint-disable-line
        }
        propsParseLoop(s, i, s[i], props);

        if (s[i] === '}' || i === s.length - 1) {
          parseAddProps(ani || at || supports || rules, i, x, props, level);
          if (ani && !level.length) {
            var selector = ani.selector;
            var c = at || supports || rules;
            ani = ani.join('');
            anims[ani]
              ? (c[c.length - 1] = c[c.length - 1].slice(0, -13) + anims[ani].slice(10) + ';}')
              : c.unshift((anims[ani] = selector) + '{' + ani + '}');
            ani = false;
          }
          if (at && !level.length) {
            (supports || rules).push(at.selector === '@font-face'
              ? at.selector + at.join('')
              : at.selector + '{' + at.join('') + '}'
            );
            at = false;
          }
          if (supports && !level.length) {
            rules.push(supports.selector + '{' + supports.join('') + '}');
            supports = false;
          }
          level.pop();
        } else if (s[i] === '{') {
          var selector$1 = s.slice(props.end, i).trim().replace(/,[\s]*/, ',& ');
          if (selector$1.indexOf('animation') === 0)
            { props.out.push(['animation', selector$1.slice(10) + ' ' + x]); }
          parseAddProps(ani || at || supports || rules, i, x, props, level);
          props.prop = props.value = props.propEnd = props.strict = null;
          selector$1.indexOf('@supports') === 0
            ? supports = Object.assign([], { selector: selector$1 })
            : selector$1[0] === '@'
              ? at = Object.assign([], { selector: ats.reduce(function (acc, fn) { return fn(acc); }, selector$1.trim()) })
              : selector$1.indexOf('animation') === 0
                ? ani = Object.assign([], { selector: '@keyframes ' + x })
                : ani
                  ? level.splice(0, level.length, selector$1)
                  : level.push(selector$1);
        }
      }

      rules.name = x;

      return rules
    }

    function parseAddProps(xs, i, name, props, level) { // eslint-disable-line
      var content = props.out.length && props.out.map(function (x) { return x.join(':'); }).join(';');
      var base = level.length && level[0][0] !== ':';
      var ani = !xs.selector || xs.selector.indexOf('@keyframes ') !== 0;
      content && xs.push(
        ((!level.length || level[0][0] !== '&') && ani
          ? (base ? '' : '.' + name) + '.' + name + (base ? ' ' : '')
          : ''
        )
        + (
          ani
            ? level.join(' ').replace(/&/g, '.' + name).replace(/ :/g, ':')
            : level.join(' ').replace(/&/g, '')
        )
        + '{' + content + ';}'
      );
      props.end = i + 1;
      props.out = [];
    }

    function propsState() {
      return {
        prop: null,
        propEnd: null,
        value:  null,
        out: [],
        strict: null,
        comma: 0,
        end: 0,
        quote: false
      }
    }

    function propsParseLoop(s, i, v, x) {
      if (!x.value && x.prop !== null && !x.strict && v === ':')
        { x.strict = true; }
      if (!x.value && x.prop !== null && x.out.length && v === ',') {
        x.value = x.comma;
        x.prop = x.out[x.out.length - 1][0];
      } else if (!x.value && x.prop === null && isIdentifier(v)) {
        x.prop = i;
      } else if (isNumber(x.prop) && !x.propEnd && !isIdentifier(v)) {
        x.propEnd = i;
      } else if (isNumber(x.propEnd) && !isNumber(x.value) && v !== ' ' && v !== '\n' && v !== ':') {
        x.value = i;
      }
      propAdd(s, i, v, x);
    }

    function propAdd(s, i, v, x) {
      if (x.value && isQuote(v))
        { x.quote = !x.quote; }

      if (!x.quote
          && x.prop !== null
          && (i === s.length - 1
            || s[i + 1] === ';'
            || s[i + 1] === '}'
            || (!x.strict
              && s[i - 1] !== ','
              && v === '\n'
            )
          )
      ) {
        x.out.push(
          propParse(
            s.slice(x.prop, x.propEnd), s.slice(x.value, i + 1).trim(), x
          )
        );
        x.end = i + (s[i + 1] === ';' ? 2 : 1);
        x.comma = x.value;
        x.value = x.prop = x.propEnd = x.strict = null;
      }
    }

    function propParse(key, value, x) {
      if (x.strict)
        { return [key, value] }

      key = props.reduce(function (acc, fn) { return fn(acc); }, key.trim());

      var result = '';
      var end = 0;
      var unit = 'px';
      for (var i = 0; i < value.length; i++) {
        if (isValueSep(value[i]) || i === value.length - 1) {
          var last = value.slice(end, i + 1).trim();
          result += ((unit && px(key))
            ? last.replace(/(^|[( ,])([-0-9.]+)([ ,)]|$)/g, '$1$2' + unit + '$3').trim()
            : last) + (value[i] === ',' ? '' : ' ');
          end = i + 1;
        }
        if (value[i] === '(' || value[i - 1] === ')') {
          unit = value.slice(end, i).indexOf('translate') === 0
            ? 'px'
            : value.slice(end, i).indexOf('rotate') === 0
              ? 'deg'
              : '';
        }
      }

      return [key, result.trim().replace(/[\n ]+/g, ' ')]
    }
  }

  var bss$1 = bss();
  bss$1.create = bss;

  return bss$1;

})));
//# sourceMappingURL=bss.js.map
